#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Jussi Määttä, 2014

# Construct a Huffman code for a given input (allowed characters: a-z and ' ')
# and output it in JSON format. This implementation uses two-character symbols
# and if the input has odd length, the final character is appended with the
# EOF symbol '#'. Hence, there are at most 27^2 + 27 = 756 possible symbols.
# In practice, not all of them end up occuring in typical text (e.g. 'zx').
# If the complete codebook of 756 symbols is desired, change the variable
# build_complete_codebook into True.
#
# Usage:
#
#     build_codebook.py < input > codebook.json

import sys
import string
import heapq
import json

build_complete_codebook = False

if build_complete_codebook:
    allowed_characters = string.ascii_lowercase + ' '
    freq = { (a + b): 0
             for a in allowed_characters
             for b in (allowed_characters + '#') }
else:
    freq = {}

input_string = sys.stdin.read()

for symbol in [ input_string[i:i+2] for i in range(0, len(input_string), 2) ]:
    if len(symbol) == 1:
        symbol = symbol + "#"

    if symbol in freq:
        freq[symbol] += 1
    else:
        freq[symbol] = 1

heap = []
for (key, value) in freq.items():
    heapq.heappush(heap, (value, key))

children = {}

while len(heap) >= 2:
    (x_count, x_char) = heapq.heappop(heap)
    (y_count, y_char) = heapq.heappop(heap)
    z_count = x_count + y_count
    z_char = x_char + "_" + y_char
    children[z_char] = (x_char, y_char)
    heapq.heappush(heap, (z_count, z_char))

(_, root) = heapq.heappop(heap)

codebook = {}
stack = [ (root, '') ]
while len(stack) > 0:
    (x, s) = stack.pop()
    if children.get(x) == None:
        codebook[x] = s
    else:
        stack.append((children[x][0], s + '0'))
        stack.append((children[x][1], s + '1'))

print(json.dumps(codebook))